<?php

namespace Litepie\Menu\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Menu\Events\MenuAction as MenuActionEvent;
use Litepie\Menu\Notifications\MenuAction as MenuActionNotification;
use Litepie\Actions\Concerns\AsAction;

class MenuAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the MenuActionEvent.
     *
     * @param   MenuActionEvent  $event
     * @return mixed
     */
    public function handle(MenuActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $menu.
     *
     * @param   MenuActionEvent  $event
     * @return void
     */
    public function create(MenuActionEvent $event)
    {
        $client = $event->$menu->client;
        Notification::send($client, new MenuActionNotification($event));
    }

    /**
     * Handle the MenuActionEvent as a listener.
     *
     * @param   MenuActionEvent  $event
     * @return mixed
     */
    public function asListener(MenuActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   MenuActionEvent  $event
     * @return bool
     */
    private function isAllowed(MenuActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
