<?php

namespace Litepie\Menu\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Menu\Events\MenuWorkflow as MenuWorkflowEvent;
use Litepie\Menu\Notifications\MenuWorkflow as MenuWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class MenuWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(MenuWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $menu is submitted.
     *
     * @param MenuWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(MenuWorkflowEvent $event)
    {
        $client = $event->$menu->client;
        Notification::send($client, new MenuWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $menu is published.
     *
     * @param MenuWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(MenuWorkflowEvent $event)
    {
        $client = $event->$menu->client;
        Notification::send($client, new MenuWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $menu is approved.
     *
     * @param MenuWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(MenuWorkflowEvent $event)
    {
        $client = $event->$menu->client;
        Notification::send($client, new MenuWorkflowNotification($event));
    }

    /**
     * Handles the $menu workflow event as a listener.
     *
     * @param MenuWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(MenuWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
